; Blur Border is a script for The GIMP
;
; Creates a blur border with a colour in between.
;
; The script is located in menu "<Image> / Script-Fu / Misc / Blur Border..."
;
; Last changed: 2011/11/13 by Franois Collard (fix error messages in Gimp 2.7)
;
; Copyright (C) 2007 Harry Phillips <script-fu@tux.com.au>
;				2010 Franois Collard <francois.collard@free.fr>
;
; --------------------------------------------------------------------
; 
; Changelog:
;  Version 1.0 (9th February 2008)
;    - Initial version
;	Version 1.1 (15th June 2010)
;	- Brightness and Contrast options added by Franois Collard
;;	- Minimal blur radius set to 0 so that:
;	0 blur 0 brightness and 0 contrast --> inner border only
;	0 blur border size --> simple solid frame around the image.
;
; --------------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.  
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, you can view the GNU General Public
; License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
; Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
; Ave, Cambridge, MA 02139, USA.
;


(define (script-fu-blur-border		theImage
					theLayer
					borderColour
					borderSize
					brightness
					contrast
					blurAmount
					blurSize
					invert)

    ;Start an undo group so the process can be undone with one undo
    (gimp-image-undo-group-start theImage)

    (let* 
    (
	;Read the current colours
	(myForeground (car (gimp-context-get-foreground)))
	(myBackground (car (gimp-context-get-background)))

	;Read the image width and height
	(imageWidth (car (gimp-image-width theImage)))
	(imageHeight (car (gimp-image-height theImage)))

	;Calculate the inner selection size
	(innerWidth (- imageWidth (* blurSize 2)))
	(innerHeight (- imageHeight (* blurSize 2)))
    )

    ;Select none
    (gimp-selection-none theImage)

    ;Select the inner square
    (gimp-image-select-rectangle theImage 2 blurSize blurSize innerWidth innerHeight)

    ;Invert the selection
    (gimp-selection-invert theImage)

    ;Blur the border
    (if (and (> blurSize 0) (> blurAmount 0))
	(plug-in-gauss 1 theImage theLayer blurAmount blurAmount 0)
	)
	;Lighten/darken border - Increase/decrease border contrast
	(if (and (> blurSize 0) (> (+ (abs brightness) (abs contrast)) 0))
	(gimp-brightness-contrast theLayer brightness contrast)
	)
	
	(if (= invert TRUE)
	(gimp-invert theLayer)
	)
	
	(if (> borderSize 0)
	(begin
    ;Invert the selection
    (gimp-selection-invert theImage)

    ;Subtract the inner amount
    (gimp-image-select-rectangle theImage 1 (+ blurSize borderSize) (+ blurSize borderSize) (- innerWidth (* borderSize 2)) (- innerHeight (* borderSize 2)))

    ;Set the background colour
    (gimp-context-set-background borderColour)

    ;Fill the border
    (gimp-edit-bucket-fill theLayer BG-BUCKET-FILL NORMAL 100 0 FALSE 0 0)
	)
	)

    ;Select none
    (gimp-selection-none theImage)

    ;Reset the background colour
    (gimp-context-set-background myBackground)

    ;Finish the undo group for the process
    (gimp-image-undo-group-end theImage)

    ;Ensure the updated image is displayed now
    (gimp-displays-flush)
))


(script-fu-register "script-fu-blur-border"
            _"<Image>/Filters/Decor/Blur Border..."
            "Gives a blurred/lightened/darkened border with a colour in between"
            "Harry Phillips"
            "Harry Phillips"
            "15 Jun 2010 - F. Collard"
            "*"
            SF-IMAGE		"Image"     0
            SF-DRAWABLE		"Drawable"  0
            SF-COLOR		_"Border color" 	'(255 255 255)
            SF-ADJUSTMENT	_"Border size"		'(10 0 200 1 10 1 0)
			SF-ADJUSTMENT	_"Brightness" 	'(80 -127 127 1 10 0 0)
			SF-ADJUSTMENT	_"Contrast" 	'(0 -127 127 1 10 0 0)
			SF-ADJUSTMENT	_"Blur amount"    	'(50 0 500 1 10 0 0)
			SF-ADJUSTMENT	_"Blur border size"	'(200 0 1024 1 10 0 0)
			SF-TOGGLE		_"Invert Border Colors"	FALSE

)            

