;; A script that compare images.
;;
;; This script load two images files and create:
;;   - a layer "ref" with the first image
;;   - a layer "out" with the second image
;;   - a layer "col-diff" containing white pixels where colors are
;;     different between the two images
;;   - a layer "alpha-diff" containing white pixels where alpha is
;;     different between the two images
;;   - a layer "diff" containing white pixel when either colors or
;;     alpha are different
;;   - an optional layer "grown-diff" where the white regions in 
;;     "diff" layer have grown by a given amount of pixels
;;
;; Copyright (C) 2011 Stephane SOPPERA
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


(define (script-fu-diff-pics infile1 infile2 ingrow)
  (let* ((img1 (car (gimp-file-load RUN-INTERACTIVE infile1 infile1)))
         (img2 (car (gimp-file-load RUN-INTERACTIVE infile2 infile2)))
	 (width (car (gimp-image-width img1)))
	 (height (car (gimp-image-height img1)))
	 (image (car (gimp-image-new width height RGB)))
	 (lay1 (car (gimp-layer-new-from-visible img1 image "ref")))
	 (lay2 (car (gimp-layer-new-from-visible img2 image "out")))
	 )
    ;; disable undo before starting
    (gimp-image-undo-disable image)
    ;; we don't need them any more after having created the new image
    (gimp-image-delete img1)
    (gimp-image-delete img2)
    ;; add the two reference layers
    (gimp-image-insert-layer image lay1 0 -1)
    (gimp-image-insert-layer image lay2 0 -1)
    ;; a function to compute the color difference between two adjacent
    ;; layers. The two layers will be lost and replace by a new layer
    ;; containing the difference.
    (define (make-diff-layer top-layer name)
      ;; change the composition mode of the second layer to difference
      (gimp-layer-set-mode top-layer DIFFERENCE-MODE)
      (let ((diff-layer (car (gimp-image-merge-down image top-layer EXPAND-AS-NECESSARY))))
	;; change layer name
	(gimp-item-set-name diff-layer name)
	;; isolate pixels that are different using a thresold
	(gimp-threshold diff-layer 0 0)
	;; invert the color so that different pixels are white, identical pixel are black
	(gimp-invert diff-layer)
	;; return the created layer
	diff-layer
	)
      )
    ;; make the color difference & alpha difference layers
    (let* ((col-diff-layer 
	    (let ((lay1-opaque (car (gimp-layer-new-from-drawable lay1 image)))
		  (lay2-opaque (car (gimp-layer-new-from-drawable lay2 image))))
	      ;; add the two layers to the image
	      (gimp-image-insert-layer image lay1-opaque 0 -1)
	      (gimp-image-insert-layer image lay2-opaque 0 -1)
	      ;; remove the alpha by filling the image with an opaque color
	      ;; while RGB channel are deactivated
	      (gimp-image-set-component-active image RED-CHANNEL FALSE)
	      (gimp-image-set-component-active image GREEN-CHANNEL FALSE)
	      (gimp-image-set-component-active image BLUE-CHANNEL FALSE)
	      (gimp-edit-fill lay1-opaque FOREGROUND-FILL)
	      (gimp-edit-fill lay2-opaque FOREGROUND-FILL)
	      (gimp-image-set-component-active image RED-CHANNEL TRUE)
	      (gimp-image-set-component-active image GREEN-CHANNEL TRUE)
	      (gimp-image-set-component-active image BLUE-CHANNEL TRUE)
	      ;; create the diff layer
	      (make-diff-layer lay2-opaque "col-diff")
	      )
	    )
	   (alpha-diff-layer
	    (let ((lay1-alpha (car (gimp-layer-new-from-drawable lay1 image)))
		  (lay2-alpha (car (gimp-layer-new-from-drawable lay2 image))))
	      ;; add the two layers to the image
	      (gimp-image-insert-layer image lay1-alpha 0 -1)
	      (gimp-image-insert-layer image lay2-alpha 0 -1)
	      ;; fill the two layer with black
	      (gimp-context-set-background '(0 0 0))
	      (gimp-drawable-fill lay1-alpha BACKGROUND-FILL)
	      (gimp-drawable-fill lay2-alpha BACKGROUND-FILL)
	      ;; paint alpha with white in layers
	      (gimp-image-select-item image CHANNEL-OP-ADD lay1)
	      (gimp-edit-fill lay1-alpha WHITE-FILL)
	      (gimp-image-select-item image CHANNEL-OP-ADD lay2)
	      (gimp-edit-fill lay2-alpha WHITE-FILL)
	      ;; clear the selection
	      (gimp-selection-none image)
	      ;; create the diff layer
	      (make-diff-layer lay2-alpha "alpha-diff")
	      )
	    )
	   ;; add alpha and diff layers to obtain final diff
	   (diff-layer
	    (let ((col-diff-layer-copy (car (gimp-layer-new-from-drawable col-diff-layer image)))
		  (alpha-diff-layer-copy (car (gimp-layer-new-from-drawable alpha-diff-layer image))))
	      ;; add the two layers to the image
	      (gimp-image-insert-layer image col-diff-layer-copy 0 -1)
	      (gimp-image-insert-layer image alpha-diff-layer-copy 0 -1)
	      ;; change the composition mode of the second layer to difference
	      (gimp-layer-set-mode alpha-diff-layer-copy ADDITION-MODE)
	      (let ((diff-layer (car (gimp-image-merge-down image alpha-diff-layer-copy EXPAND-AS-NECESSARY))))
		;; change layer name
		(gimp-item-set-name diff-layer "diff")
		diff-layer
		)
	      )
	    )
	   )
      ;; hide col and alpha difference layers
      (gimp-item-set-visible col-diff-layer FALSE)
      (gimp-item-set-visible alpha-diff-layer FALSE)
      ;; create a new diff layer using grow
      (if (> ingrow 0)
      	  (let ((grown-diff-layer (car (gimp-layer-new-from-drawable diff-layer image))))
	    ;; change layer name
	    (gimp-item-set-name grown-diff-layer "grown-diff")
      	    ;; add the layer to the image
      	    (gimp-image-insert-layer image grown-diff-layer 0 -1)
      	    ;; fill the layer in black
      	    (gimp-context-set-background '(0 0 0))
      	    (gimp-item-fill grown-diff-layer BACKGROUND-FILL)
      	    ;; select all white pixels of the diff
      	    (gimp-by-color-select diff-layer '(255 255 255) 0 CHANNEL-OP-REPLACE FALSE FALSE 0 FALSE)
      	    ;; grow the selection
      	    (gimp-selection-grow image ingrow)
      	    ;; paint the selection in white
      	    (gimp-edit-fill grown-diff-layer WHITE-FILL)
      	    ;; clear the selection
      	    (gimp-selection-none image)
      	    )
      	  )
      )
    
    ;; enable the undo for the image
    (gimp-image-undo-enable image)
    ;; display the created image
    (gimp-display-new image)
    )
  )

(script-fu-register
  "script-fu-diff-pics"
  _"<Toolbox>/File/Compare images..."
  "Compare two images"
  "Stephane SOPPERA"
  "copyright 2011, Stephane SOPPERA"
  "2011/12/04"
  ""
  SF-FILENAME "Ref" ""
  SF-FILENAME "Out" "" 
  SF-ADJUSTMENT "Add grow layer"    '(0 0 1000 1 10 0 1)
)

